@@ -26,17 +26,26 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: file.c,v 2.37 1996/05/21 03:20:56 jrs Exp $
+ *	$Id: file.c,v 1.10 1999/04/21 15:57:38 wichert Exp $
  */
 
 #include "defs.h"
 
 #include <dirent.h>
-#include <sys/stat.h>
+
+#ifdef linux
+#  include <asm/stat.h>
+#  define stat libc_stat
+#  include <sys/stat.h>
+#  undef stat
+#else
+#  include <sys/stat.h>
+#endif
+
 #include <fcntl.h>
 
 #ifdef SVR4
-#include <sys/cred.h>
+#  include <sys/cred.h>
 #endif /* SVR4 */
 
 #include <sys/vfs.h>
@@ -90,6 +99,16 @@ struct xlat openmodes[] = {
 #ifdef O_PRIV
 	{ O_PRIV,	"O_PRIV"	},
 #endif
+#ifdef O_DIRECT
+   { O_DIRECT, "O_DIRECT"  },
+#endif
+#ifdef O_LARGEFILE
+   { O_LARGEFILE,  "O_LARGEFILE"   },
+#endif
+#ifdef O_DIRECTORY
+   { O_DIRECTORY,  "O_DIRECTORY"   },
+#endif
+
 #ifdef FNDELAY
 	{ FNDELAY,	"FNDELAY"	},
 #endif
@@ -149,6 +168,44 @@ struct tcb *tcp;
 	return 0;
 }
 
+#ifdef LINUXSPARC
+struct xlat openmodessol[] = {
+	{ 0,		"O_RDWR"	},
+	{ 1,		"O_RDONLY"	},
+	{ 2,		"O_WRONLY"	},
+	{ 0x80,		"O_NONBLOCK"	},
+	{ 8,		"O_APPEND"	},
+	{ 0x100,	"O_CREAT"	},
+	{ 0x200,	"O_TRUNC"	},
+	{ 0x400,	"O_EXCL"	},
+	{ 0x800,	"O_NOCTTY"	},
+	{ 0x10,		"O_SYNC"	},
+	{ 0x40,		"O_DSYNC"	},
+	{ 0x8000,	"O_RSYNC"	},
+	{ 4,		"O_NDELAY"	},
+	{ 0x1000,	"O_PRIV"	},
+	{ 0,		NULL		},
+};
+
+int
+solaris_open(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		printpath(tcp, tcp->u_arg[0]);
+		tprintf(", ");
+		/* flags */
+		printflags(openmodessol, tcp->u_arg[1] + 1);
+		if (tcp->u_arg[1] & 0x100) {
+			/* mode */
+			tprintf(", %#lo", tcp->u_arg[2]);
+		}
+	}
+	return 0;
+}
+
+#endif
+
 int
 sys_creat(tcp)
 struct tcb *tcp;
@@ -213,11 +270,40 @@ struct tcb *tcp;
 			tprintf("%lu, ", tcp->u_arg[1]);
 		else
 			tprintf("%ld, ", tcp->u_arg[1]);
-		printxval(whence, tcp->u_arg[2], "L_???");
+		printxval(whence, tcp->u_arg[2], "SEEK_???");
 	}
 	return RVAL_UDECIMAL;
 }
 
+#ifdef linux
+int
+sys_llseek (tcp)
+struct tcb *tcp;
+{
+    if (entering(tcp)) {
+	if (tcp->u_arg[4] == SEEK_SET)
+	    tprintf("%ld, %llu, ", tcp->u_arg[0],
+		    (((unsigned long long int) tcp->u_arg[1]) << 32
+		     | (unsigned long) tcp->u_arg[2]));
+	else
+	    tprintf("%ld, %lld, ", tcp->u_arg[0],
+		    (((long long int) tcp->u_arg[1]) << 32
+		     | (unsigned long) tcp->u_arg[2]));
+    }
+    else {
+	if (syserror(tcp))
+	    tprintf("%#lx, ", tcp->u_arg[3]);
+	else {
+	    long long int off;
+	    umove(tcp, tcp->u_arg[3], &off);
+	    tprintf("{%lld}, ", off);
+	}
+	printxval(whence, tcp->u_arg[4], "SEEK_???");
+    }
+    return 0;
+}
+#endif
+
 int
 sys_truncate(tcp)
 struct tcb *tcp;
@@ -294,12 +380,39 @@ time_t t;
 	return buf;
 }
 
+#ifdef LINUXSPARC
+typedef struct {
+        int     tv_sec;
+        int     tv_nsec;
+} timestruct_t;
+
+struct solstat {
+        unsigned        st_dev;
+        int             st_pad1[3];     /* network id */
+        unsigned        st_ino;
+        unsigned        st_mode;
+        unsigned        st_nlink;
+        unsigned        st_uid;
+        unsigned        st_gid;
+        unsigned        st_rdev;
+        int             st_pad2[2];
+        int             st_size;
+        int             st_pad3;        /* st_size, off_t expansion */
+        timestruct_t    st_atime;
+        timestruct_t    st_mtime;
+        timestruct_t    st_ctime;
+        int             st_blksize;
+        int             st_blocks;
+        char            st_fstype[16];
+        int             st_pad4[8];     /* expansion area */
+};
+
 static void
-printstat(tcp, addr)
+printstatsol(tcp, addr)
 struct tcb *tcp;
 int addr;
 {
-	struct stat statbuf;
+	struct solstat statbuf;
 
 	if (!addr) {
 		tprintf("NULL");
@@ -315,37 +428,27 @@ int addr;
 	}
 	if (!abbrev(tcp)) {
 		tprintf("{st_dev=makedev(%lu, %lu), st_ino=%lu, st_mode=%s, ",
-			(unsigned long) major(statbuf.st_dev),
-			(unsigned long) minor(statbuf.st_dev),
+			(unsigned long) ((statbuf.st_dev >> 18) & 0x3fff),
+			(unsigned long) (statbuf.st_dev & 0x3ffff),
 			(unsigned long) statbuf.st_ino,
 			sprintmode(statbuf.st_mode));
 		tprintf("st_nlink=%lu, st_uid=%lu, st_gid=%lu, ",
 			(unsigned long) statbuf.st_nlink,
 			(unsigned long) statbuf.st_uid,
 			(unsigned long) statbuf.st_gid);
-#ifdef HAVE_ST_BLKSIZE
 		tprintf("st_blksize=%lu, ", (unsigned long) statbuf.st_blksize);
-#endif /* HAVE_ST_BLKSIZE */
-#ifdef HAVE_ST_BLOCKS
 		tprintf("st_blocks=%lu, ", (unsigned long) statbuf.st_blocks);
-#endif /* HAVE_ST_BLOCKS */
 	}
 	else
 		tprintf("{st_mode=%s, ", sprintmode(statbuf.st_mode));
 	switch (statbuf.st_mode & S_IFMT) {
 	case S_IFCHR: case S_IFBLK:
-#ifdef HAVE_ST_RDEV
 		tprintf("st_rdev=makedev(%lu, %lu), ",
-			(unsigned long) major(statbuf.st_rdev),
-			(unsigned long) minor(statbuf.st_rdev));
-#else /* !HAVE_ST_RDEV */
-		tprintf("st_size=makedev(%lu, %lu), ",
-			(unsigned long) major(statbuf.st_size),
-			(unsigned long) minor(statbuf.st_size));
-#endif /* !HAVE_ST_RDEV */
+			(unsigned long) ((statbuf.st_rdev >> 18) & 0x3fff),
+			(unsigned long) (statbuf.st_rdev & 0x3ffff));
 		break;
 	default:
-		tprintf("st_size=%lu, ", statbuf.st_size);
+		tprintf("st_size=%u, ", statbuf.st_size);
 		break;
 	}
 	if (!abbrev(tcp)) {
@@ -356,6 +459,145 @@ int addr;
 	else
 		tprintf("...}");
 }
+#endif /* LINUXSPARC */
+
+static void
+realprintstat(tcp, statbuf)
+struct tcb *tcp;
+struct stat *statbuf;
+{
+    if (!abbrev(tcp)) {
+	    tprintf("{st_dev=makedev(%lu, %lu), st_ino=%lu, st_mode=%s, ",
+		    (unsigned long) major(statbuf->st_dev),
+		    (unsigned long) minor(statbuf->st_dev),
+		    (unsigned long) statbuf->st_ino,
+		    sprintmode(statbuf->st_mode));
+	    tprintf("st_nlink=%lu, st_uid=%lu, st_gid=%lu, ",
+		    (unsigned long) statbuf->st_nlink,
+		    (unsigned long) statbuf->st_uid,
+		    (unsigned long) statbuf->st_gid);
+#ifdef HAVE_ST_BLKSIZE
+	    tprintf("st_blksize=%lu, ", (unsigned long) statbuf->st_blksize);
+#endif /* HAVE_ST_BLKSIZE */
+#ifdef HAVE_ST_BLOCKS
+	    tprintf("st_blocks=%lu, ", (unsigned long) statbuf->st_blocks);
+#endif /* HAVE_ST_BLOCKS */
+    }
+    else
+	    tprintf("{st_mode=%s, ", sprintmode(statbuf->st_mode));
+    switch (statbuf->st_mode & S_IFMT) {
+    case S_IFCHR: case S_IFBLK:
+#ifdef HAVE_ST_RDEV
+	    tprintf("st_rdev=makedev(%lu, %lu), ",
+		    (unsigned long) major(statbuf->st_rdev),
+		    (unsigned long) minor(statbuf->st_rdev));
+#else /* !HAVE_ST_RDEV */
+	    tprintf("st_size=makedev(%lu, %lu), ",
+		    (unsigned long) major(statbuf->st_size),
+		    (unsigned long) minor(statbuf->st_size));
+#endif /* !HAVE_ST_RDEV */
+	    break;
+    default:
+	    tprintf("st_size=%lu, ", statbuf->st_size);
+	    break;
+    }
+    if (!abbrev(tcp)) {
+	    tprintf("st_atime=%s, ", sprinttime(statbuf->st_atime));
+	    tprintf("st_mtime=%s, ", sprinttime(statbuf->st_mtime));
+	    tprintf("st_ctime=%s}", sprinttime(statbuf->st_ctime));
+    }
+    else
+	    tprintf("...}");
+}
+
+
+static void
+printstat(tcp, addr)
+struct tcb *tcp;
+int addr;
+{
+	struct stat statbuf;
+
+#ifdef LINUXSPARC
+ 	if (current_personality == 1) {
+ 		printstatsol(tcp, addr);
+ 		return;
+ 	}
+#endif /* LINUXSPARC */
+
+	if (!addr) {
+		tprintf("NULL");
+		return;
+	}
+	if (syserror(tcp) || !verbose(tcp)) {
+		tprintf("%#x", addr);
+		return;
+	}
+	if (umove(tcp, addr, &statbuf) < 0) {
+		tprintf("{...}");
+		return;
+	}
+
+	realprintstat(tcp, &statbuf);
+}
+
+#ifdef linux
+static void
+convertoldstat(oldbuf, newbuf)
+const struct __old_kernel_stat *oldbuf;
+struct stat *newbuf;
+{
+    newbuf->st_dev=oldbuf->st_dev;
+    newbuf->st_ino=oldbuf->st_ino;
+    newbuf->st_mode=oldbuf->st_mode;
+    newbuf->st_nlink=oldbuf->st_nlink;
+    newbuf->st_uid=oldbuf->st_uid;
+    newbuf->st_gid=oldbuf->st_gid;
+    newbuf->st_rdev=oldbuf->st_rdev;
+    newbuf->st_size=oldbuf->st_size;
+    newbuf->st_atime=oldbuf->st_atime;
+    newbuf->st_mtime=oldbuf->st_mtime;
+    newbuf->st_ctime=oldbuf->st_ctime;
+    newbuf->st_blksize=0;	/* not supported in old_stat */
+    newbuf->st_blocks=0;		/* not supported in old_stat */
+}
+#endif
+
+
+#ifdef linux
+static void
+printoldstat(tcp, addr)
+struct tcb *tcp;
+int addr;
+{
+	struct __old_kernel_stat statbuf;
+	struct stat newstatbuf;
+
+#ifdef LINUXSPARC
+ 	if (current_personality == 1) {
+ 		printstatsol(tcp, addr);
+ 		return;
+ 	}
+#endif /* LINUXSPARC */
+
+	if (!addr) {
+		tprintf("NULL");
+		return;
+	}
+	if (syserror(tcp) || !verbose(tcp)) {
+		tprintf("%#x", addr);
+		return;
+	}
+	if (umove(tcp, addr, &statbuf) < 0) {
+		tprintf("{...}");
+		return;
+	}
+
+	convertoldstat(&statbuf, &newstatbuf);
+	realprintstat(tcp, &newstatbuf);
+}
+#endif
+
 
 int
 sys_stat(tcp)
@@ -370,6 +612,21 @@ struct tcb *tcp;
 	return 0;
 }
 
+#ifdef linux
+int
+sys_oldstat(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		printpath(tcp, tcp->u_arg[0]);
+		tprintf(", ");
+	} else {
+		printoldstat(tcp, tcp->u_arg[1]);
+	}
+	return 0;
+}
+#endif
+
 int
 sys_fstat(tcp)
 struct tcb *tcp;
@@ -382,6 +639,20 @@ struct tcb *tcp;
 	return 0;
 }
 
+#ifdef linux
+int
+sys_oldfstat(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp))
+		tprintf("%ld, ", tcp->u_arg[0]);
+	else {
+		printoldstat(tcp, tcp->u_arg[1]);
+	}
+	return 0;
+}
+#endif
+
 int
 sys_lstat(tcp)
 struct tcb *tcp;
@@ -395,7 +666,23 @@ struct tcb *tcp;
 	return 0;
 }
 
-#ifdef SVR4
+#ifdef linux
+int
+sys_oldlstat(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		printpath(tcp, tcp->u_arg[0]);
+		tprintf(", ");
+	} else {
+		printoldstat(tcp, tcp->u_arg[1]);
+	}
+	return 0;
+}
+#endif
+
+
+#if defined(SVR4) || defined(LINUXSPARC)
 
 int
 sys_xstat(tcp)
@@ -449,9 +736,15 @@ struct tcb *tcp;
 		tprintf(", %s", sprintmode(mode));
 		switch (mode & S_IFMT) {
 		case S_IFCHR: case S_IFBLK:
+#ifdef LINUXSPARC
+			tprintf(", makedev(%lu, %lu)",
+				(unsigned long) ((tcp->u_arg[3] >> 18) & 0x3fff),
+				(unsigned long) (tcp->u_arg[3] & 0x3ffff));
+#else		
 			tprintf(", makedev(%lu, %lu)",
 				(unsigned long) major(tcp->u_arg[3]),
 				(unsigned long) minor(tcp->u_arg[3]));
+#endif				
 			break;
 		default:
 			break;
@@ -460,9 +753,9 @@ struct tcb *tcp;
 	return 0;
 }
 
-#endif /* SVR4 */
+#endif /* SVR4 || LINUXSPARC */
 
-#ifdef LINUX
+#ifdef linux
 
 static struct xlat fsmagic[] = {
 	{ 0xef51,	"EXT2_OLD_SUPER_MAGIC"	},
@@ -479,7 +772,7 @@ static struct xlat fsmagic[] = {
 	{ 0,		NULL			},
 };
 
-#endif /* LINUX */
+#endif /* linux */
 
 #ifndef SVR4
 
@@ -488,7 +781,7 @@ sprintfstype(magic)
 int magic;
 {
 	static char buf[32];
-#ifdef LINUX
+#ifdef linux
 	char *s;
 
 	s = xlookup(fsmagic, magic);
@@ -496,7 +789,7 @@ int magic;
 		sprintf(buf, "\"%s\"", s);
 		return buf;
 	}
-#endif /* LINUX */
+#endif /* linux */
 	sprintf(buf, "%#x", magic);
 	return buf;
 }
@@ -518,20 +811,23 @@ long addr;
 	}
 #ifdef ALPHA
 
-	tprintf("{f_type=%s, f_fsize=%u, f_fbsize=%u, f_blocks=%u, f_bfree=%u, ",
+	tprintf("{f_type=%s, f_fbsize=%u, f_blocks=%u, f_bfree=%u, ",
 		sprintfstype(statbuf.f_type),
-		statbuf.f_fsize,statbuf.f_bsize, statbuf.f_blocks, statbuf.f_bfree);
+		statbuf.f_bsize, statbuf.f_blocks, statbuf.f_bfree);
 	tprintf("f_bavail=%u, f_files=%u, f_ffree=%u, f_namelen=%u}",
 		statbuf.f_bavail,statbuf.f_files, statbuf.f_ffree, statbuf.f_namelen);
 #else /* !ALPHA */
 	tprintf("{f_type=%s, f_bsize=%lu, f_blocks=%lu, f_bfree=%lu, ",
 		sprintfstype(statbuf.f_type),
-		statbuf.f_bsize, statbuf.f_blocks, statbuf.f_bfree);
+		(unsigned long)statbuf.f_bsize,
+		(unsigned long)statbuf.f_blocks,
+		(unsigned long)statbuf.f_bfree);
 	tprintf("f_files=%lu, f_ffree=%lu",
-		statbuf.f_files, statbuf.f_ffree);
-#ifdef LINUX
-	tprintf(", f_namelen=%lu}", statbuf.f_namelen);
-#endif /* LINUX */
+		(unsigned long)statbuf.f_files,
+		(unsigned long)statbuf.f_ffree);
+#ifdef linux
+	tprintf(", f_namelen=%lu}", (unsigned long)statbuf.f_namelen);
+#endif /* linux */
 #endif /* !ALPHA */
 	tprintf("}");
 }
@@ -561,8 +857,7 @@ struct tcb *tcp;
 	return 0;
 }
 
-#ifdef LINUX
-#ifdef ALPHA
+#if defined(linux) && defined(__alpha)
 
 int
 osf_statfs(tcp)
@@ -590,8 +885,7 @@ struct tcb *tcp;
 	}
 	return 0;
 }
-#endif /* ALPHA */
-#endif /* LINUX */
+#endif /* linux && __alpha */
 
 #endif /* !SVR4 */
 
@@ -844,6 +1138,13 @@ struct tcb *tcp;
 		tprintf(", %s", sprintmode(mode));
 		switch (mode & S_IFMT) {
 		case S_IFCHR: case S_IFBLK:
+#ifdef LINUXSPARC
+			if (current_personality == 1)
+			tprintf(", makedev(%lu, %lu)",
+				(unsigned long) ((tcp->u_arg[2] >> 18) & 0x3fff),
+				(unsigned long) (tcp->u_arg[2] & 0x3ffff));
+			else
+#endif	
 			tprintf(", makedev(%lu, %lu)",
 				(unsigned long) major(tcp->u_arg[2]),
 				(unsigned long) minor(tcp->u_arg[2]));
@@ -876,7 +1177,7 @@ struct tcb *tcp;
 	return 0;
 }
 
-#ifdef LINUX
+#ifdef linux
 
 static void
 printdir(tcp, addr)
@@ -894,14 +1195,6 @@ long addr;
 		return;
 	}
 	tprintf("{d_ino=%ld, ", (unsigned long) d.d_ino);
-#ifndef LINUX
-	/* This contains garbage under Linux.  */
-	tprintf("d_off=%d, ", d.d_off);
-#endif /* !LINUX */
-#ifndef LINUX
-	/* No point in printing this out since the syscall returns it. */
-	tprintf("d_reclen=%u, ", d.d_reclen);
-#endif /* !LINUX */
 	tprintf("d_name=");
 	printpathn(tcp, (long) ((struct dirent *) addr)->d_name, d.d_reclen);
 	tprintf("}");
@@ -925,7 +1218,7 @@ struct tcb *tcp;
 	return 0;
 }
 
-#endif /* LINUX */
+#endif /* linux */
 
 int
 sys_getdents(tcp)
@@ -956,14 +1249,14 @@ struct tcb *tcp;
 		tprintf("{");
 	for (i = 0; i < len;) {
 		struct dirent *d = (struct dirent *) &buf[i];
-#ifdef LINUX
+#ifdef linux
 		if (!abbrev(tcp)) {
 			tprintf("%s{d_ino=%lu, d_off=%lu, ",
 				i ? " " : "", d->d_ino, d->d_off);
 			tprintf("d_reclen=%u, d_name=\"%s\"}",
 				d->d_reclen, d->d_name);
 		}
-#endif /* LINUX */
+#endif /* linux */
 #ifdef SVR4
 		if (!abbrev(tcp)) {
 			tprintf("%s{d_ino=%lu, d_off=%lu, ",
@@ -993,6 +1286,23 @@ struct tcb *tcp;
 	return 0;
 }
 
+#ifdef linux
+
+int
+sys_getcwd(tcp)
+struct tcb *tcp;
+{
+    if (exiting(tcp)) {
+	if (syserror(tcp))
+	    tprintf("%#lx", tcp->u_arg[0]);
+	else
+	    printstr(tcp, tcp->u_arg[0], tcp->u_arg[1]);
+	tprintf(", %lu", tcp->u_arg[1]);
+    }
+    return 0;
+}
+#endif /* linux */
+
 #ifdef HAVE_SYS_ASYNCH_H
 
 int
