@@ -26,7 +26,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: io.c,v 2.29 1996/05/19 06:40:36 jrs Exp $
+ *	$Id: io.c,v 1.1.1.1 1999/02/19 00:22:12 wichert Exp $
  */
 
 #include "defs.h"
@@ -141,6 +141,40 @@ struct tcb *tcp;
 
 #ifdef SVR4
 
+int
+sys_pread(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		tprintf("%ld, ", tcp->u_arg[0]);
+	} else {
+		if (syserror(tcp))
+			tprintf("%#lx", tcp->u_arg[1]);
+		else
+			printstr(tcp, tcp->u_arg[1], tcp->u_rval);
+		tprintf(", %lu, %llu", tcp->u_arg[2],
+				(((unsigned long long) tcp->u_arg[4]) << 32
+				 | tcp->u_arg[3]));
+	}
+	return 0;
+}
+
+int
+sys_pwrite(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		tprintf("%ld, ", tcp->u_arg[0]);
+		printstr(tcp, tcp->u_arg[1], tcp->u_arg[2]);
+		tprintf(", %lu, %llu", tcp->u_arg[2],
+				(((unsigned long long) tcp->u_arg[4]) << 32
+				 | tcp->u_arg[3]));
+	}
+	return 0;
+}
+#endif /* SVR4 */
+
+#ifdef LINUX
 int
 sys_pread(tcp)
 struct tcb *tcp;
@@ -169,7 +203,7 @@ struct tcb *tcp;
 	return 0;
 }
 
-#endif /* SVR4 */
+#endif /* LINUX */
 
 int
 sys_ioctl(tcp)
