@@ -25,7 +25,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: syscallent.h,v 2.25 1996/05/21 03:21:06 jrs Exp $
+ *	$Id: syscallent.h,v 1.1.1.1 1999/02/19 00:23:02 wichert Exp $
  */
 
 	{ 0,	0,	sys_setup,		"setup"		}, /* 0 */
@@ -44,7 +44,7 @@
 	{ 1,	0,	sys_time,		"time"		}, /* 13 */
 	{ 3,	TF,	sys_mknod,		"mknod"		}, /* 14 */
 	{ 2,	TF,	sys_chmod,		"chmod"		}, /* 15 */
-	{ 3,	TF,	sys_chown,		"chown"		}, /* 16 */
+	{ 3,	TF,	sys_chown,		"lchown"		}, /* 16 */
 	{ 0,	0,	sys_break,		"break"		}, /* 17 */
 	{ 2,	TF,	sys_oldstat,		"oldstat"	}, /* 18 */
 	{ 3,	0,	sys_lseek,		"lseek"		}, /* 19 */
@@ -118,7 +118,7 @@
 	{ 1,	TF,	sys_swapon,		"swapon"	}, /* 87 */
 	{ 3,	0,	sys_reboot,		"reboot"	}, /* 88 */
 	{ 3,	0,	sys_readdir,		"readdir"	}, /* 89 */
-	{ 1,	0,	sys_mmap,		"mmap"		}, /* 90 */
+	{ 6,	0,	sys_mmap,		"mmap"		}, /* 90 */
 	{ 2,	0,	sys_munmap,		"munmap"	}, /* 91 */
 	{ 2,	TF,	sys_truncate,		"truncate"	}, /* 92 */
 	{ 2,	0,	sys_ftruncate,		"ftruncate"	}, /* 93 */
@@ -141,7 +141,7 @@
 	{ 1,	0,	sys_iopl,		"iopl"		}, /* 110 */
 	{ 0,	0,	sys_vhangup,		"vhangup"	}, /* 111 */
 	{ 0,	0,	sys_idle,		"idle"		}, /* 112 */
-	{ 1,	0,	sys_vm86,		"vm86"		}, /* 113 */
+	{ 1,	0,	sys_vm86old,		"vm86old"		}, /* 113 */
 	{ 4,	TP,	sys_wait4,		"wait4"		}, /* 114 */
 	{ 1,	0,	sys_swapoff,		"swapoff"	}, /* 115 */
 	{ 1,	0,	sys_sysinfo,		"sysinfo"	}, /* 116 */
@@ -192,41 +192,41 @@
 	{ 5,	0,	sys_sched_rr_get_interval,"sched_rr_get_interval"}, /* 161 */
 	{ 5,	0,	sys_nanosleep,		"nanosleep"	}, /* 162 */
 	{ 5,	0,	sys_mremap,		"mremap"	}, /* 163 */
-	{ 5,	0,	printargs,		"SYS_164"	}, /* 164 */
-	{ 5,	0,	printargs,		"SYS_165"	}, /* 165 */
-	{ 5,	0,	printargs,		"SYS_166"	}, /* 166 */
-	{ 5,	0,	printargs,		"SYS_167"	}, /* 167 */
-	{ 5,	0,	printargs,		"SYS_168"	}, /* 168 */
-	{ 5,	0,	printargs,		"SYS_169"	}, /* 169 */
-	{ 5,	0,	printargs,		"SYS_170"	}, /* 170 */
-	{ 5,	0,	printargs,		"SYS_171"	}, /* 171 */
-	{ 5,	0,	printargs,		"SYS_172"	}, /* 172 */
-	{ 5,	0,	printargs,		"SYS_173"	}, /* 173 */
-	{ 5,	0,	printargs,		"SYS_174"	}, /* 174 */
-	{ 5,	0,	printargs,		"SYS_175"	}, /* 175 */
-	{ 5,	0,	printargs,		"SYS_176"	}, /* 176 */
-	{ 5,	0,	printargs,		"SYS_177"	}, /* 177 */
-	{ 5,	0,	printargs,		"SYS_178"	}, /* 178 */
-	{ 5,	0,	printargs,		"SYS_179"	}, /* 179 */
+	{ 3,	0,	sys_setresuid,		"setresuid"	}, /* 164 */
+	{ 3,	0,	sys_getresuid,		"getresuid"	}, /* 165 */
+	{ 5,	0,	printargs,		"vm86"		}, /* 166 */
+	{ 5,	0,	sys_query_module,	"query_module"	}, /* 167 */
+	{ 3,	0,	sys_poll,		"poll"		}, /* 168 */
+	{ 5,	0,	printargs,		"nfsservctl"	}, /* 169 */
+	{ 3,	0,	sys_setresgid,		"setresgid"	}, /* 170 */
+	{ 3,	0,	sys_getresgid,		"getresgid"	}, /* 171 */
+	{ 5,	0,	printargs,		"prctl"		}, /* 172 */
+	{ 1,	TS,	printargs,		"rt_sigreturn"	}, /* 173 */
+	{ 4,	TS,	sys_rt_sigaction,	"rt_sigaction"  }, /* 174 */
+	{ 4,	TS,	sys_rt_sigprocmask,	"rt_sigprocmask"}, /* 175 */
+	{ 2,	TS,	sys_rt_sigpending,	"rt_sigpending"	}, /* 176 */
+	{ 4,	TS,	sys_rt_sigtimedwait,	"rt_sigtimedwait"}, /* 177 */
+	{ 3,	TS,	sys_rt_sigqueueinfo,    "rt_sigqueueinfo"}, /* 178 */
+	{ 2,	TS,	sys_rt_sigsuspend,	"rt_sigsuspend"	}, /* 179 */
 
-	{ 8,	0,	printargs,		"socket_subcall"}, /* 180 */
-	{ 3,	TN,	sys_socket,		"socket"	}, /* 181 */
-	{ 3,	TN,	sys_bind,		"bind"		}, /* 182 */
-	{ 3,	TN,	sys_connect,		"connect"	}, /* 183 */
-	{ 2,	TN,	sys_listen,		"listen"	}, /* 184 */
-	{ 3,	TN,	sys_accept,		"accept"	}, /* 185 */
-	{ 3,	TN,	sys_getsockname,	"getsockname"	}, /* 186 */
-	{ 3,	TN,	sys_getpeername,	"getpeername"	}, /* 187 */
-	{ 4,	TN,	sys_socketpair,		"socketpair"	}, /* 188 */
-	{ 4,	TN,	sys_send,		"send"		}, /* 189 */
-	{ 4,	TN,	sys_recv,		"recv"		}, /* 180 */
-	{ 6,	TN,	sys_sendto,		"sendto"	}, /* 191 */
-	{ 6,	TN,	sys_recvfrom,		"recvfrom"	}, /* 192 */
-	{ 2,	TN,	sys_shutdown,		"shutdown"	}, /* 193 */
-	{ 5,	TN,	sys_setsockopt,		"setsockopt"	}, /* 194 */
-	{ 5,	TN,	sys_getsockopt,		"getsockopt"	}, /* 195 */
-	{ 5,	TN,	sys_sendmsg,		"sendmsg"	}, /* 196 */
-	{ 5,	TN,	sys_recvmsg,		"recvmsg"	}, /* 197 */
+	{ 5,	TF,	sys_pread,		"pread"		}, /* 180 */
+	{ 5,	TF,	sys_pwrite,		"pwrite"	}, /* 181 */
+	{ 3,	TF,	sys_chown,		"chown"		}, /* 182 */
+	{ 2,	0,	sys_getcwd,		"getcwd"	}, /* 183 */
+	{ 5,	0,	printargs,		"capget"	}, /* 184 */
+	{ 5,	0,	printargs,		"capset"	}, /* 185 */
+	{ 5,	TS,	sys_sigaltstack,	"sigaltstack"	}, /* 186 */
+	{ 5,	TF,	printargs,		"sendfile"	}, /* 187 */
+	{ 5,	0,	printargs,		"SYS_188"	}, /* 188 */
+	{ 5,	0,	printargs,		"SYS_189"	}, /* 189 */
+	{ 0,	TP,	sys_vfork,		"vfork"		}, /* 190 */
+	{ 5,	0,	printargs,		"SYS_191"	}, /* 191 */
+	{ 5,	0,	printargs,		"SYS_192"	}, /* 192 */
+	{ 5,	0,	printargs,		"SYS_193"	}, /* 193 */
+	{ 5,	0,	printargs,		"SYS_194"	}, /* 194 */
+	{ 5,	0,	printargs,		"SYS_195"	}, /* 195 */
+	{ 5,	0,	printargs,		"SYS_196"	}, /* 196 */
+	{ 5,	0,	printargs,		"SYS_197"	}, /* 197 */
 	{ 5,	0,	printargs,		"SYS_198"	}, /* 198 */
 	{ 5,	0,	printargs,		"SYS_199"	}, /* 199 */
 
@@ -260,3 +260,25 @@
 	{ 5,	0,	printargs,		"SYS_227"	}, /* 227 */
 	{ 5,	0,	printargs,		"SYS_228"	}, /* 228 */
 	{ 5,	0,	printargs,		"SYS_229"	}, /* 229 */
+
+	{ 8,	0,	printargs,		"socket_subcall"}, /* 230 */
+	{ 3,	TN,	sys_socket,		"socket"	}, /* 231 */
+	{ 3,	TN,	sys_bind,		"bind"		}, /* 232 */
+	{ 3,	TN,	sys_connect,		"connect"	}, /* 233 */
+	{ 2,	TN,	sys_listen,		"listen"	}, /* 234 */
+	{ 3,	TN,	sys_accept,		"accept"	}, /* 235 */
+	{ 3,	TN,	sys_getsockname,	"getsockname"	}, /* 236 */
+	{ 3,	TN,	sys_getpeername,	"getpeername"	}, /* 237 */
+	{ 4,	TN,	sys_socketpair,		"socketpair"	}, /* 238 */
+	{ 4,	TN,	sys_send,		"send"		}, /* 239 */
+	{ 4,	TN,	sys_recv,		"recv"		}, /* 240 */
+	{ 6,	TN,	sys_sendto,		"sendto"	}, /* 241 */
+	{ 6,	TN,	sys_recvfrom,		"recvfrom"	}, /* 242 */
+	{ 2,	TN,	sys_shutdown,		"shutdown"	}, /* 243 */
+	{ 5,	TN,	sys_setsockopt,		"setsockopt"	}, /* 244 */
+	{ 5,	TN,	sys_getsockopt,		"getsockopt"	}, /* 245 */
+	{ 5,	TN,	sys_sendmsg,		"sendmsg"	}, /* 246 */
+	{ 5,	TN,	sys_recvmsg,		"recvmsg"	}, /* 247 */
+	{ 5,	0,	printargs,		"SYS_248"	}, /* 248 */
+	{ 5,	0,	printargs,		"SYS_249"	}, /* 249 */
+
