@@ -26,7 +26,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: process.c,v 2.36 1996/05/19 06:40:36 jrs Exp $
+ *	$Id: process.c,v 1.7 1999/03/29 23:23:13 nate Exp $
  */
 
 #include "defs.h"
@@ -42,8 +42,18 @@
 #include <signal.h>
 #ifdef SUNOS4
 #include <machine/reg.h>
+#endif /* SUNOS4 */
+
+#if __GLIBC__ == 2 && __GLIBC_MINOR__ >= 1 && (defined(I386) || defined(M68K))
+# include <sys/reg.h>
+# define PTRACE_PEEKUSR PTRACE_PEEKUSER
+# define PTRACE_POKEUSR PTRACE_POKEUSER
 #endif
 
+#ifdef LINUX
+#include <linux/ptrace.h>
+#endif /* LINUX */
+
 #ifdef HAVE_PRCTL
 #include <sys/prctl.h>
 #endif
@@ -52,22 +62,54 @@
 #define WCOREDUMP(status) ((status) & 0200)
 #endif
 
-#ifdef HAVE_PRCTL
+/* WTA: this has `&& !defined(LINUXSPARC)', this seems unneeded though? */
+#if defined(HAVE_PRCTL)
 static struct xlat prctl_options[] = {
+#ifdef PR_MAXPROCS
 	{ PR_MAXPROCS,		"PR_MAXPROCS"		},
+#endif
+#ifdef PR_ISBLOCKED
 	{ PR_ISBLOCKED,		"PR_ISBLOCKED"		},
+#endif
+#ifdef PR_SETSTACKSIZE
 	{ PR_SETSTACKSIZE,	"PR_SETSTACKSIZE"	},
+#endif
+#ifdef PR_GETSTACKSIZE
 	{ PR_GETSTACKSIZE,	"PR_GETSTACKSIZE"	},
+#endif
+#ifdef PR_MAXPPROCS
 	{ PR_MAXPPROCS,		"PR_MAXPPROCS"		},
+#endif
+#ifdef PR_UNBLKONEXEC
 	{ PR_UNBLKONEXEC,	"PR_UNBLKONEXEC"	},
+#endif
+#ifdef PR_ATOMICSIM
 	{ PR_ATOMICSIM,		"PR_ATOMICSIM"		},
+#endif
+#ifdef PR_SETEXITSIG
 	{ PR_SETEXITSIG,	"PR_SETEXITSIG"		},
+#endif
+#ifdef PR_RESIDENT
 	{ PR_RESIDENT,		"PR_RESIDENT"		},
+#endif
+#ifdef PR_ATTACHADDR
 	{ PR_ATTACHADDR,	"PR_ATTACHADDR"		},
+#endif
+#ifdef PR_DETACHADDR
 	{ PR_DETACHADDR,	"PR_DETACHADDR"		},
+#endif
+#ifdef PR_TERMCHILD
 	{ PR_TERMCHILD,		"PR_TERMCHILD"		},
+#endif
+#ifdef PR_GETSHMASK
 	{ PR_GETSHMASK,		"PR_GETSHMASK"		},
+#endif
+#ifdef PR_GETNSHARE
 	{ PR_GETNSHARE,		"PR_GETNSHARE"		},
+#endif
+#if defined(PR_SET_PDEATHSIG)
+	{ PR_SET_PDEATHSIG,	"PR_SET_PDEATHSIG"	},
+#endif
 	{ 0,			NULL			},
 };
 
@@ -80,8 +122,10 @@ struct tcb *tcp;
 	if (entering(tcp)) {
 		printxval(prctl_options, tcp->u_arg[0], "PR_???");
 		switch (tcp->u_arg[0]) {
+#ifdef PR_GETNSHARE
 		case PR_GETNSHARE:
 			break;
+#endif
 		default:
 			for (i = 1; i < tcp->u_nargs; i++)
 				tprintf(", %#lx", tcp->u_arg[i]);
@@ -138,7 +182,7 @@ struct tcb *tcp;
 	return 0;
 }
 
-#ifndef LINUX
+#if !defined(LINUX)
 
 int
 sys_getdomainname(tcp)
@@ -241,13 +285,30 @@ struct tcb *tcp;
 {
 	struct tcb *tcpchild;
 	int pid;
-	int vforking = 0;
+	int dont_follow = 0;
 
 #ifdef SYS_vfork
-	vforking = (tcp->scno == EXEC_vfork);
+	if (tcp->scno == SYS_vfork) {
+#if defined(I386) && defined(LINUX)
+		/* Attempt to make vfork into fork, which we can follow. */
+		if (!followvfork || 
+		    ptrace(PTRACE_POKEUSR, tcp->pid, 
+			   (void *)(ORIG_EAX * 4), SYS_fork) < 0) 
+			dont_follow = 1;
+		
+#else
+		dont_follow = 1;
+#endif
+	}
+#endif
+#ifdef SYS_clone
+	/* clone can do many things, not all of which we know how to handle.
+	   Don't do it for now. */
+	if (tcp->scno == SYS_clone)
+		dont_follow = 1;
 #endif
 	if (entering(tcp)) {
-		if (!followfork || vforking)
+		if (!followfork || dont_follow)
 			return 0;
 		if (nprocs == MAX_PROCS) {
 			tcp->flags &= ~TCB_FOLLOWFORK;
@@ -333,7 +394,7 @@ struct tcb *tcp;
 
 #endif /* !SVR4 */
 
-#ifdef SUNOS4
+#if defined(SUNOS4) || defined(LINUX)
 
 int
 sys_vfork(tcp)
@@ -344,7 +405,7 @@ struct tcb *tcp;
 	return 0;
 }
 
-#endif /* SUNOS4 */
+#endif /* SUNOS4 || LINUX */
 
 #ifndef LINUX
 
@@ -410,6 +471,48 @@ struct tcb *tcp;
 	return 0;
 }
 
+int
+sys_getresuid(tcp)
+    struct tcb *tcp;
+{
+	if (exiting(tcp)) {
+		uid_t res[3];
+		if (umoven(tcp, tcp->u_arg[0], sizeof(pid_t),
+					(char *) &res[0]) < 0
+				|| umoven(tcp, tcp->u_arg[2], sizeof(pid_t),
+					(char *) &res[1]) < 0
+				|| umoven(tcp, tcp->u_arg[2], sizeof(pid_t),
+					(char *) &res[2]) < 0)
+			return -1;
+		tprintf("ruid %lu, euid %lu, suid %lu",
+				(unsigned long) res[0],
+				(unsigned long) res[1],
+				(unsigned long) res[2]);
+	}
+	return 0;
+}
+
+int
+sys_getresgid(tcp)
+struct tcb *tcp;
+{
+	if (exiting(tcp)) {
+		uid_t res[3];
+		if (umoven(tcp, tcp->u_arg[0], sizeof(pid_t),
+					(char *) &res[0]) < 0
+				|| umoven(tcp, tcp->u_arg[2], sizeof(pid_t),
+					(char *) &res[1]) < 0
+				|| umoven(tcp, tcp->u_arg[2], sizeof(pid_t),
+					(char *) &res[2]) < 0)
+			return -1;
+		tprintf("rgid %lu, egid %lu, sgid %lu",
+				(unsigned long) res[0],
+				(unsigned long) res[1],
+				(unsigned long) res[2]);
+	}
+	return 0;
+}
+
 #endif /* LINUX */
 
 int
@@ -436,6 +539,34 @@ struct tcb *tcp;
 	return 0;
 }
 
+#ifdef LINUX
+int
+sys_setresuid(tcp)
+     struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		tprintf("ruid %u, euid %u, suid %u",
+				(uid_t) tcp->u_arg[0],
+				(uid_t) tcp->u_arg[1],
+				(uid_t) tcp->u_arg[2]);
+	}
+	return 0;
+}
+int
+sys_setresgid(tcp)
+     struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		tprintf("rgid %u, egid %u, sgid %u",
+				(uid_t) tcp->u_arg[0],
+				(uid_t) tcp->u_arg[1],
+				(uid_t) tcp->u_arg[2]);
+	}
+	return 0;
+}
+
+#endif /* LINUX */
+
 int
 sys_setgroups(tcp)
 struct tcb *tcp;
@@ -582,8 +713,13 @@ char *envp[];
 {
 	int i;
 
+#ifdef ARM
+	if (!(qual_flags[SYS_execve - __NR_SYSCALL_BASE] & QUAL_TRACE))
+		return;
+#else
 	if (!(qual_flags[SYS_execve] & QUAL_TRACE))
 		return;
+#endif /* !ARM */
 	printleader(tcp);
 	tprintf("execve(");
 	string_quote(program);
@@ -611,6 +747,8 @@ long addr;
 	int max = max_strlen / 2;
 
 	for (sep = ""; --max >= 0; sep = ", ") {
+		if (!abbrev(tcp))
+			max++;
 		if (umove(tcp, addr, &cp) < 0) {
 			tprintf("%#lx", addr);
 			return;
@@ -683,15 +821,15 @@ struct tcb *tcp;
 		else if (abbrev(tcp))
 			printargc(", [/* %d var%s */]", tcp, tcp->u_arg[2]);
 		else {
-			tprintf("[");
+			tprintf(", [");
 			printargv(tcp, tcp->u_arg[2]);
 			tprintf("]");
 		}
 	}
 #ifdef LINUX
-#ifdef ALPHA
+#if defined(ALPHA) || defined(SPARC) || defined(POWERPC)
 	tcp->flags |= TCB_WAITEXECVE;
-#endif /* ALPHA */
+#endif /* ALPHA || SPARC || POWERPC */
 #endif /* LINUX */
 	return 0;
 }
@@ -752,10 +890,10 @@ int status;
 	 */
 	if (WIFSTOPPED(status))
 		tprintf("[WIFSTOPPED(s) && WSTOPSIG(s) == %s]",
-			signalent[WSTOPSIG(status)]);
+			signame(WSTOPSIG(status)));
 	else if WIFSIGNALED(status)
 		tprintf("[WIFSIGNALED(s) && WTERMSIG(s) == %s%s]",
-			signalent[WTERMSIG(status)],
+			signame(WTERMSIG(status)),
 			WCOREDUMP(status) ? " && WCOREDUMP(s)" : "");
 	else if WIFEXITED(status) {
 		tprintf("[WIFEXITED(s) && WEXITSTATUS(s) == %d]",
@@ -816,7 +954,8 @@ internal_wait(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		if (tcp->nchildren > 0) {
+		/* WTA: fix bug with hanging children */
+		if (!(tcp->u_arg[2] & WNOHANG) && tcp->nchildren > 0) {
 			/* There are traced children */
 			tcp->flags |= TCB_SUSPENDED;
 			tcp->waitpid = tcp->u_arg[0];
@@ -1144,9 +1283,9 @@ struct tcb *tcp;
 				uname.release, uname.version);
 			tprintf("machine=\"%s\"", uname.machine);
 #ifdef LINUX
-#ifndef ALPHA
+#ifndef __GLIBC__
 			tprintf(", domainname=\"%s\"", uname.domainname);
-#endif /* ALPHA */
+#endif /* __GLIBC__ */
 #endif /* LINUX */
 			tprintf("}");
 		}
@@ -1213,6 +1352,50 @@ static
 #endif /* !SUNOS4_KERNEL_ARCH_KLUDGE */
 struct xlat struct_user_offsets[] = {
 #ifdef LINUX
+#ifdef SPARC
+	/* XXX No support for these offsets yet. */
+#elif defined(POWERPC)
+	{ 4*PT_R0,		"4*PT_R0"	},
+	{ 4*PT_R1,		"4*PT_R1"	},
+	{ 4*PT_R2,		"4*PT_R2"	},
+	{ 4*PT_R3,		"4*PT_R3"	},
+	{ 4*PT_R4,		"4*PT_R4"	},
+	{ 4*PT_R5,		"4*PT_R5"	},
+	{ 4*PT_R6,		"4*PT_R6"	},
+	{ 4*PT_R7,		"4*PT_R7"	},
+	{ 4*PT_R8,		"4*PT_R8"	},
+	{ 4*PT_R9,		"4*PT_R9"	},
+	{ 4*PT_R10,		"4*PT_R10"	},
+	{ 4*PT_R11,		"4*PT_R11"	},
+	{ 4*PT_R12,		"4*PT_R12"	},
+	{ 4*PT_R13,		"4*PT_R13"	},
+	{ 4*PT_R14,		"4*PT_R14"	},
+	{ 4*PT_R15,		"4*PT_R15"	},
+	{ 4*PT_R16,		"4*PT_R16"	},
+	{ 4*PT_R17,		"4*PT_R17"	},
+	{ 4*PT_R18,		"4*PT_R18"	},
+	{ 4*PT_R19,		"4*PT_R19"	},
+	{ 4*PT_R20,		"4*PT_R20"	},
+	{ 4*PT_R21,		"4*PT_R21"	},
+	{ 4*PT_R22,		"4*PT_R22"	},
+	{ 4*PT_R23,		"4*PT_R23"	},
+	{ 4*PT_R24,		"4*PT_R24"	},
+	{ 4*PT_R25,		"4*PT_R25"	},
+	{ 4*PT_R26,		"4*PT_R26"	},
+	{ 4*PT_R27,		"4*PT_R27"	},
+	{ 4*PT_R28,		"4*PT_R28"	},
+	{ 4*PT_R29,		"4*PT_R29"	},
+	{ 4*PT_R30,		"4*PT_R30"	},
+	{ 4*PT_R31,		"4*PT_R31"	},
+	{ 4*PT_NIP,		"4*PT_NIP"	},
+	{ 4*PT_MSR,		"4*PT_MSR"	},
+	{ 4*PT_ORIG_R3,	 "4*PT_ORIG_R3"	},
+	{ 4*PT_CTR,		"4*PT_CTR"	},
+	{ 4*PT_LNK,		"4*PT_LNK"	},
+	{ 4*PT_XER,		"4*PT_XER"	},
+	{ 4*PT_CCR,		"4*PT_CCR"	},
+	{ 4*PT_FPR0,	"4*PT_FPR0"	},
+#else	
 #ifdef ALPHA
 	{ 0,		"r0"					},
 	{ 1,		"r1"					},
@@ -1337,13 +1520,16 @@ struct xlat struct_user_offsets[] = {
 	{ uoff(signal),		"offsetof(struct user, signal)"		},
 	{ uoff(reserved),	"offsetof(struct user, reserved)"	},
 	{ uoff(u_ar0),		"offsetof(struct user, u_ar0)"		},
+#ifndef ARM
 	{ uoff(u_fpstate),	"offsetof(struct user, u_fpstate)"	},
+#endif
 	{ uoff(magic),		"offsetof(struct user, magic)"		},
 	{ uoff(u_comm),		"offsetof(struct user, u_comm)"		},
 #ifdef I386
 	{ uoff(u_debugreg),	"offsetof(struct user, u_debugreg)"	},
 #endif /* I386 */
 #endif /* !ALPHA */
+#endif /* !POWERPC/!SPARC */
 #endif /* LINUX */
 #ifdef SUNOS4
 	{ uoff(u_pcb),		"offsetof(struct user, u_pcb)"		},
