@@ -24,7 +24,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: sock.c,v 2.23 1996/05/19 06:40:37 jrs Exp $
+ *	$Id: sock.c,v 1.1.1.1 1999/02/19 00:22:29 wichert Exp $
  */
 
 #include "defs.h"
@@ -35,6 +35,10 @@
 #include <sys/sockio.h>
 #endif
 
+#ifdef ALPHA
+#include <ioctls.h>
+#endif
+
 int
 sock_ioctl(tcp, code, arg)
 struct tcb *tcp;
