@@ -26,7 +26,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: strace.c,v 2.45 1996/05/19 06:40:37 jrs Exp $
+ *	$Id: strace.c,v 1.2 1999/03/29 23:23:13 nate Exp $
  */
 
 #include "defs.h"
@@ -40,6 +40,7 @@
 #include <sys/stat.h>
 #include <pwd.h>
 #include <grp.h>
+#include <string.h>
 
 #ifdef SVR4
 #include <sys/stropts.h>
@@ -865,7 +866,7 @@ int sig;
 	}
 #endif /* LINUX */
 
-#ifdef SUNOS4
+#if defined(SUNOS4)
 	/* PTRACE_DETACH won't respect `sig' argument, so we post it here. */
 	if (sig && kill(tcp->pid, sig) < 0)
 		perror("detach: kill");
@@ -1313,7 +1314,7 @@ trace()
 			if (!cflag && (qual_flags[what] & QUAL_SIGNAL)) {
 				printleader(tcp);
 				tprintf("--- %s (%s) ---",
-					signalent[what], strsignal(what));
+					signame(what), strsignal(what));
 				printtrailer(tcp);
 			}
 			break;
@@ -1425,7 +1426,7 @@ trace()
 			    && (qual_flags[WTERMSIG(status)] & QUAL_SIGNAL)) {
 				printleader(tcp);
 				tprintf("+++ killed by %s +++",
-					signalent[WTERMSIG(status)]);
+					signame(WTERMSIG(status)));
 				printtrailer(tcp);
 			}
 			droptcb(tcp);
@@ -1448,7 +1449,7 @@ trace()
 		}
 		if (debug)
 			fprintf(stderr, "pid %u stopped, [%s]\n",
-				pid, signalent[WSTOPSIG(status)]);
+				pid, signame(WSTOPSIG(status)));
 
 		if (tcp->flags & TCB_STARTUP) {
 			/*
@@ -1508,7 +1509,7 @@ trace()
 			    && (qual_flags[WSTOPSIG(status)] & QUAL_SIGNAL)) {
 				printleader(tcp);
 				tprintf("--- %s (%s) ---",
-					signalent[WSTOPSIG(status)],
+					signame(WSTOPSIG(status)),
 					strsignal(WSTOPSIG(status)));
 				printtrailer(tcp);
 			}
