@@ -24,12 +24,26 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: stream.c,v 2.11 1996/05/21 03:20:57 jrs Exp $
+ *	$Id: stream.c,v 1.2 1999/04/18 23:30:29 wichert Exp $
  */
 
 #include "defs.h"
 
-#ifdef HAVE_SYS_STREAM_H
+#if defined(HAVE_SYS_STREAM_H) || defined(linux)
+
+#if defined(linux)
+#include <sys/poll.h>
+
+#define RS_HIPRI 1
+struct strbuf {
+        int     maxlen;                 /* no. of bytes in buffer */
+        int     len;                    /* no. of bytes returned */
+        char    *buf;                   /* pointer to data */
+};
+#define MORECTL 1
+#define MOREDATA 2
+
+#else /* linux */
 
 #include <stropts.h>
 #include <poll.h>
@@ -37,6 +51,8 @@
 #include <sys/stream.h>
 #include <sys/tihdr.h>
 
+#endif /* linux */
+
 #ifdef HAVE_SYS_TIUSER_H
 #include <sys/tiuser.h>
 #include <sys/sockmod.h>
@@ -48,13 +64,6 @@ static struct xlat msgflags[] = {
 	{ 0,		NULL		},
 };
 
-#if 0
-static struct xlat getmsgflags[] = {
-	{ MORECTL,	"MORECTL"	},
-	{ MOREDATA,	"MOREDATA"	},
-	{ 0,		NULL		},
-};
-#endif
 
 static void
 printstrbuf(tcp, sbp, getting)
@@ -240,6 +249,7 @@ struct tcb *tcp;
 
 #endif /* HAVE_PUTPMSG */
 
+
 static struct xlat pollflags[] = {
 	{ POLLIN,	"POLLIN"	},
 	{ POLLPRI,	"POLLPRI"	},
@@ -311,15 +321,19 @@ struct tcb *tcp;
 			tprintf("]");
 		}
 		tprintf(", %d, ", nfds);
+#ifdef INFTIM
 		if (tcp->u_arg[2] == INFTIM)
 			tprintf("INFTIM");
 		else
+#endif
 			tprintf("%ld", tcp->u_arg[2]);
 		free(pollp);
 	}
 	return 0;
 }
 
+#ifndef linux
+
 static struct xlat stream_flush_options[] = {
 	{ FLUSHR,	"FLUSHR"	},
 	{ FLUSHW,	"FLUSHW"	},
@@ -793,4 +807,7 @@ int code, arg;
 	}
 }
 
-#endif /* HAVE_SYS_STREAM_H */
+#endif /* linux */
+
+#endif /* LINUXSPARC && linux */
+
