@@ -26,7 +26,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: syscall.c,v 2.42 1996/05/20 05:39:41 jrs Exp $
+ *	$Id: syscall.c,v 1.4 1999/04/18 22:50:51 wichert Exp $
  */
 
 #include "defs.h"
@@ -37,6 +37,13 @@
 #include <sys/user.h>
 #include <sys/syscall.h>
 #include <sys/param.h>
+#if __GLIBC__ == 2 && __GLIBC_MINOR__ >= 1 && (defined(I386) || defined(M68K))
+# include <sys/reg.h>
+#endif
+
+#ifdef LINUX
+#include <linux/ptrace.h>
+#endif /* LINUX */
 
 #ifndef SYS_ERRLIST_DECLARED
 extern int sys_nerr;
@@ -56,6 +63,13 @@ extern char *sys_errlist[];
 #ifndef ENOIOCTLCMD
 #define ENOIOCTLCMD	515	/* No ioctl command */
 #endif
+#ifndef NSIG
+#define NSIG 32
+#endif
+#ifdef ARM
+#undef NSIG
+#define NSIG 32
+#endif
 #endif /* LINUX */
 
 #include "syscall.h"
@@ -83,7 +97,7 @@ int nsyscalls1 = sizeof sysent1 / sizeof sysent1[0];
 struct sysent sysent2[] = {
 #include "syscallent2.h"
 };
-int nsyscalls1 = sizeof sysent2 / sizeof sysent2[0];
+int nsyscalls2 = sizeof sysent2 / sizeof sysent2[0];
 #endif /* SUPPORTED_PERSONALITIES >= 3 */
 
 struct sysent *sysent;
@@ -149,7 +163,7 @@ set_personality(int personality)
 #endif /* SUPPORTED_PERSONALITIES >= 2 */
 
 #if SUPPORTED_PERSONALITIES >= 3
-	case 1:
+	case 2:
 		errnoent = errnoent2;
 		nerrnos = nerrnos2;
 		sysent = sysent2;
@@ -238,7 +252,7 @@ char *s;
 	if (strncmp(s, "SIG", 3) == 0)
 		s += 3;
 	for (i = 0; i <= NSIG; i++) {
-		if (strcmp(s, signalent[i] + 3) == 0)
+		if (strcmp(s, signame(i) + 3) == 0)
 			return i;
 	}
 	return -1;
@@ -352,7 +366,11 @@ struct tcb *tcp;
 		return;
 	if (tcp->u_arg[0] < 0 || tcp->u_arg[0] >= MAX_QUALS)
 		return;
+#ifdef __arm__
+	switch (tcp->scno + __NR_SYSCALL_BASE) {
+#else
 	switch (tcp->scno) {
+#endif
 	case SYS_read:
 #ifdef SYS_recv
 	case SYS_recv:
@@ -380,6 +398,47 @@ enum subcall_style { shift_style, deref_style, mask_style };
 
 #if !(defined(LINUX) && defined(ALPHA))
 
+const int socket_map [] = {
+	       /* SYS_SOCKET      */ 97,
+	       /* SYS_BIND        */ 104,
+	       /* SYS_CONNECT     */ 98,
+	       /* SYS_LISTEN      */ 106,
+	       /* SYS_ACCEPT      */ 99,
+	       /* SYS_GETSOCKNAME */ 150,
+	       /* SYS_GETPEERNAME */ 141,
+	       /* SYS_SOCKETPAIR  */ 135,
+	       /* SYS_SEND        */ 101,
+	       /* SYS_RECV        */ 102,
+	       /* SYS_SENDTO      */ 133,
+	       /* SYS_RECVFROM    */ 125,
+	       /* SYS_SHUTDOWN    */ 134,
+	       /* SYS_SETSOCKOPT  */ 105,
+	       /* SYS_GETSOCKOPT  */ 118,
+	       /* SYS_SENDMSG     */ 114,
+	       /* SYS_RECVMSG     */ 113
+};
+
+void
+sparc_socket_decode (struct tcb *tcp)
+{
+	volatile long addr;
+	volatile int i, n;
+
+	if (tcp->u_arg [0] < 1 || tcp->u_arg [0] > sizeof(socket_map)/sizeof(int)+1){
+		return;
+	}
+	tcp->scno = socket_map [tcp->u_arg [0]-1];
+	n = tcp->u_nargs = sysent [tcp->scno].nargs;
+	addr = tcp->u_arg [1];
+	for (i = 0; i < n; i++){
+	        int arg;
+		if (umoven (tcp, addr, sizeof (arg), (void *) &arg) < 0)
+			arg = 0;
+		tcp->u_arg [i] = arg;
+		addr += sizeof (arg);
+	}
+}
+
 static void
 decode_subcall(tcp, subcall, nsubcalls, style)
 struct tcb *tcp;
@@ -436,13 +495,19 @@ struct tcb *tcp;
 	 * correctly support following forks in the presence of tracing
 	 * qualifiers.
 	 */
+#ifdef __arm__
+	switch (tcp->scno + __NR_SYSCALL_BASE) {
+#else
 	switch (tcp->scno) {
-
+#endif
 #ifdef SYS_fork
 	case SYS_fork:
 #endif
 #ifdef SYS_vfork
 	case SYS_vfork:
+#endif
+#ifdef SYS_clone
+	case SYS_clone:
 #endif
 		internal_fork(tcp);
 		break;
@@ -486,21 +551,25 @@ struct tcb *tcp;
 {
 	int sys_res;
 	struct timeval tv;
-	long scno;
+	long scno = 0;
 #ifdef LINUX
-#ifdef I386
+#if defined (I386)
 	long eax;
-#else /* !I386 */
-#ifdef M68K
+#elif defined (POWERPC)
+	long result,flags;
+#elif defined (M68K)
 	int d0;
-#else /* !M68K */
-#ifdef ALPHA
+#elif defined (ARM)
+	int r0;
+#elif defined (ALPHA)
 	long r0;
 	long a3;
-#endif /* ALPHA */
-#endif /* M68K */
-#endif /* !I386 */
+#elif defined (SPARC)
+	struct pt_regs regs;
+	unsigned long trap;
+#endif 
 #endif /* LINUX */
+
 #ifndef SVR4
 	int pid = tcp->pid;
 #endif /* !SVR4 */
@@ -509,15 +578,30 @@ struct tcb *tcp;
 	if (dtime && (tcp->flags & TCB_INSYSCALL))
 		gettimeofday(&tv, NULL);
 #ifdef LINUX
-#ifdef I386
+#if defined (POWERPC)
+	if (upeek(pid, 4*PT_R0, &scno) < 0)
+		return -1;
+	if (!(tcp->flags & TCB_INSYSCALL)) {
+		/* Check if we return from execve. */
+		if (scno == 0 && (tcp->flags & TCB_WAITEXECVE)) {
+			tcp->flags &= ~TCB_WAITEXECVE;
+			return 0;
+		}
+	}
+#elif defined (I386)
 	if (upeek(pid, 4*ORIG_EAX, &scno) < 0)
 		return -1;
-#else /* !I386 */
-#ifdef M68K
+#elif defined (ARM)
+	{ 
+	    long pc;
+	    upeek(pid, 4*15, &pc);
+	    umoven(tcp, pc-4, 4, (char *)&scno);
+	    scno &= 0x000fffff;
+	}
+#elif defined (M68K)
 	if (upeek(pid, 4*PT_ORIG_D0, &scno) < 0)
 		return -1;
-#else /* !M68K */
-#ifdef ALPHA
+#elif defined (ALPHA)
 	if (upeek(pid, REG_A3, &a3) < 0)
 		return -1;
 
@@ -547,9 +631,65 @@ struct tcb *tcp;
 		if (upeek(pid, REG_R0, &r0) < 0)
 			return -1;
 	}
-#endif /* ALPHA */
-#endif /* !M68K */
-#endif /* !I386 */
+#elif defined (SPARC)
+	/* Everything we need is in the current register set. */
+	if (ptrace(PTRACE_GETREGS,pid,(char *)&regs,0) < 0)
+		return -1;
+
+	memmove (&regs.u_regs [1], &regs.u_regs [0],
+		 sizeof (regs.u_regs) - sizeof (regs.u_regs [0]));
+
+        /* If we are entering, then disassemble the syscall trap. */
+	if (!(tcp->flags & TCB_INSYSCALL)) {
+		/* Retrieve the syscall trap instruction. */
+		errno = 0;
+		trap = ptrace(PTRACE_PEEKTEXT,pid,(char *)regs.pc,0);
+		if (errno)
+			return -1;
+
+		/* Disassemble the trap to see what personality to use. */
+		switch (trap) {
+		case 0x91d02010:
+			/* Linux/SPARC syscall trap. */
+			set_personality(0);
+			break;
+		case 0x91d02000:
+			/* SunOS syscall trap. (pers 1) */
+			fprintf(stderr,"syscall: SunOS no support\n");
+			return -1;
+		case 0x91d02008:
+			/* Solaris 2.x syscall trap. (per 2) */
+			set_personality(1);
+			break; 
+		case 0x91d02009:
+			/* NetBSD/FreeBSD syscall trap. */
+			fprintf(stderr,"syscall: NetBSD/FreeBSD not supported\n");
+			return -1;
+		case 0x91d02027:
+			/* Solaris 2.x gettimeofday */
+			set_personality(1);
+			break;
+		default:
+			/* Unknown syscall trap. */
+			if(tcp->flags & TCB_WAITEXECVE) {
+				tcp->flags &= ~TCB_WAITEXECVE;
+				return 0;
+			}
+			fprintf(stderr,"syscall: unknown syscall trap %08x %08x\n", trap, regs.pc);
+			return -1;
+		}
+
+		/* Extract the system call number from the registers. */
+		if (trap == 0x91d02027)
+			scno = 156;
+		else
+			scno = regs.u_regs[UREG_G1];
+		if (scno == 0) {
+			scno = regs.u_regs[UREG_I0];
+			memmove (&regs.u_regs[UREG_I0], &regs.u_regs[UREG_I1], 7*sizeof(regs.u_regs[0]));
+		}
+	}
+#endif 
 #endif /* LINUX */
 #ifdef SUNOS4
 	if (upeek(pid, uoff(u_arg[7]), &scno) < 0)
@@ -609,7 +749,7 @@ struct tcb *tcp;
 	}
 #endif /* SUNOS4 */
 #ifdef LINUX
-#ifdef I386
+#if defined (I386)
 	if (upeek(pid, 4*EAX, &eax) < 0)
 		return -1;
 	if (eax != -ENOSYS && !(tcp->flags & TCB_INSYSCALL)) {
@@ -617,8 +757,15 @@ struct tcb *tcp;
 			fprintf(stderr, "stray syscall exit: eax = %ld\n", eax);
 		return 0;
 	}
-#else /* !I386 */
-#ifdef M68K
+#elif defined (POWERPC)
+# define SO_MASK 0x10000000
+	if (upeek(pid, 4*PT_CCR, &flags) < 0)
+		return -1;
+	if (upeek(pid, 4*PT_R3, &result) < 0)
+		return -1;
+	if (flags & SO_MASK)
+		result = -result;
+#elif defined (M68K)
 	if (upeek(pid, 4*PT_D0, &d0) < 0)
 		return -1;
 	if (d0 != -ENOSYS && !(tcp->flags & TCB_INSYSCALL)) {
@@ -626,8 +773,16 @@ struct tcb *tcp;
 			fprintf(stderr, "stray syscall exit: d0 = %ld\n", d0);
 		return 0;
 	}
-#endif /* M68K */
-#endif /* !I386 */
+#elif defined (ARM)
+	if (upeek(pid, 4*0, (long *)&r0) < 0)
+		return -1;
+	if ( 0 && r0 != -ENOSYS && !(tcp->flags & TCB_INSYSCALL)) {
+		if (debug)
+			fprintf(stderr, "stray syscall exit: d0 = %ld\n", r0);
+		return 0;
+	}
+#else
+#endif
 #endif /* LINUX */
 
 	if (tcp->flags & TCB_INSYSCALL) {
@@ -644,6 +799,16 @@ struct tcb *tcp;
 			u_error = 0;
 		}
 #else /* !I386 */
+#ifdef POWERPC
+		if (result && (unsigned) -result < nerrnos) {
+			tcp->u_rval = -1;
+			u_error = -result;
+		}
+		else {
+			tcp->u_rval = result;
+			u_error = 0;
+		}
+#else /* !POWERPC */
 #ifdef M68K
 		if (d0 && (unsigned) -d0 < nerrnos) {
 			tcp->u_rval = -1;
@@ -654,6 +819,16 @@ struct tcb *tcp;
 			u_error = 0;
 		}
 #else /* !M68K */
+#ifdef ARM
+		if (r0 && (unsigned) -r0 < nerrnos) {
+			tcp->u_rval = -1;
+			u_error = -r0;
+		}
+		else {
+			tcp->u_rval = r0;
+			u_error = 0;
+		}
+#else /* !ARM */
 #ifdef ALPHA
 		if (a3) {
 			tcp->u_rval = -1;
@@ -663,9 +838,22 @@ struct tcb *tcp;
 			tcp->u_rval = r0;
 			u_error = 0;
 		}
+#else /* !ALPHA */
+#ifdef SPARC
+		if (regs.psr & PSR_C) {
+			tcp->u_rval = -1;
+			u_error = regs.u_regs[UREG_I0];
+		}
+		else {
+			tcp->u_rval = regs.u_regs[UREG_I0];
+			u_error = 0;
+		}
+#endif /* SPARC */
 #endif /* ALPHA */
-#endif /* !M68K */
-#endif /* !I386 */
+#endif /* ARM */
+#endif /* M68K */
+#endif /* POWERPC */
+#endif /* I386 */
 #endif /* LINUX */
 #ifdef SUNOS4
 		/* get error code from user struct */
@@ -845,16 +1033,37 @@ struct tcb *tcp;
 	/* Entering system call */
 	tcp->scno = scno;
 #ifdef LINUX
-#ifdef ALPHA
+#if defined (ALPHA)
 	{
 		int i;
 		tcp->u_nargs = sysent[tcp->scno].nargs;
 		for (i = 0; i < tcp->u_nargs; i++) {
+			/* WTA: if scno is out-of-bounds this will bomb. Add range-check
+			 * for scno somewhere above here!
+			 */
 			if (upeek(pid, REG_A0+i, &tcp->u_arg[i]) < 0)
 				return -1;
 		}
 	}
-#else /* !ALPHA */
+#elif defined (POWERPC)
+	{
+		int i;
+		tcp->u_nargs = sysent[tcp->scno].nargs;
+		for (i = 0; i < tcp->u_nargs; i++) {
+			if (upeek(pid, (i==0) ? (4*PT_ORIG_R3) : ((i+PT_R3)*4), &tcp->u_arg[i]) < 0)
+				return -1;
+		}
+	}
+#elif defined (SPARC)
+	{
+		int i, offset;
+	         
+	        offset = UREG_I0;
+		tcp->u_nargs = sysent[tcp->scno].nargs;
+		for (i = 0; i < tcp->u_nargs; i++)
+			tcp->u_arg[i] = regs.u_regs[offset + i];
+	}
+#else 
 	{
 		int i;
 		tcp->u_nargs = sysent[tcp->scno].nargs;
@@ -863,7 +1072,7 @@ struct tcb *tcp;
 				return -1;
 		}
 	}
-#endif
+#endif 
 #endif /* LINUX */
 #ifdef SUNOS4
 	{
@@ -921,9 +1130,13 @@ struct tcb *tcp;
 #endif /* !HAVE_PR_SYSCALL */
 #endif /* !MIPS */
 #endif /* SVR4 */
+#ifdef __arm__
+	switch (tcp->scno + __NR_SYSCALL_BASE) {
+#else
 	switch (tcp->scno) {
+#endif
 #ifdef LINUX
-#ifndef ALPHA
+#if !defined (ALPHA) && !defined(SPARC)
 	case SYS_socketcall:
 		decode_subcall(tcp, SYS_socket_subcall,
 			SYS_socket_nsubcalls, deref_style);
@@ -932,7 +1145,12 @@ struct tcb *tcp;
 		decode_subcall(tcp, SYS_ipc_subcall,
 			SYS_ipc_nsubcalls, shift_style);
 		break;
-#endif /* ALPHA */
+#endif /* !ALPHA && !SPARC */
+#ifdef SPARC
+	case SYS_socketcall:
+		sparc_socket_decode (tcp);
+		break;
+#endif
 #endif /* LINUX */
 #ifdef SVR4
 #ifdef SYS_pgrpsys_subcall
@@ -1048,6 +1266,15 @@ struct tcb *tcp;
 {
 	long val = -1;
 
+#ifdef LINUX
+#ifdef SPARC
+	struct pt_regs regs;
+	if (ptrace(PTRACE_GETREGS,tcp->pid,(char *)&regs,0) < 0)
+		return -1;
+	val = regs.u_regs[UREG_I1];
+#endif /* SPARC */
+#endif /* LINUX */
+
 #ifdef SUNOS4
 	if (upeek(tcp->pid, uoff(u_rval2), &val) < 0)
 		return -1;
