@@ -26,29 +26,36 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: system.c,v 2.31 1996/05/19 06:40:37 jrs Exp $
+ *	$Id: system.c,v 1.1.1.1 1999/02/19 00:22:47 wichert Exp $
  */
 
 #include "defs.h"
 
 #ifdef LINUX
+#define _LINUX_SOCKET_H
+
+#define MS_RDONLY   1  /* Mount read-only */
+#define MS_NOSUID   2  /* Ignore suid and sgid bits */
+#define MS_NODEV    4  /* Disallow access to device special files */
+#define MS_NOEXEC   8  /* Disallow program execution */
+#define MS_SYNCHRONOUS 16  /* Writes are synced at once */
+#define MS_REMOUNT 32  /* Alter flags of a mounted FS */
 
-#include <linux/fs.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <linux/nfs.h>
-#include <linux/nfs_mount.h>
 
-#include <linux/unistd.h>
-
-#ifdef __NR_personality
+#ifdef LINUX
 /* Workaround for kernel namespace pollution. */
 #define sys_personality kernel_sys_personality
 #include <linux/personality.h>
 #undef sys_personality
 #endif /* __NR_personality */
 
+#ifdef __NR_capget
+#include <linux/capability.h>
+#endif
+
 static struct xlat mount_flags[] = {
 	{ MS_RDONLY,	"MS_RDONLY"	},
 	{ MS_NOSUID,	"MS_NOSUID"	},
@@ -84,6 +91,9 @@ static struct xlat personality_options[] = {
 #ifdef PER_LINUX
 	{ PER_LINUX,	"PER_LINUX"	},
 #endif
+#ifdef PER_LINUX_32BIT
+	{ PER_LINUX_32BIT,	"PER_LINUX"	},
+#endif
 #ifdef PER_SVR4
 	{ PER_SVR4,	"PER_SVR4"	},
 #endif
@@ -821,5 +831,56 @@ struct tcb *tcp;
 
 #endif /* !MIPS */
 
-
 #endif /* SVR4 */
+
+#ifdef __NR_capget
+int
+sys_capget(tcp)
+struct tcb *tcp;
+{
+	cap_user_header_t       arg0;
+	cap_user_data_t         arg1;
+
+	if(!entering(tcp)) {
+		arg0 = (cap_user_header_t)tcp->u_arg[0];
+		arg1 = (cap_user_data_t)tcp->u_arg[1];
+		tprintf("{%lx, %d}, ", (unsigned long)arg0->version, arg0->pid);
+		tprintf("{%lx, %lx, %lx}", (unsigned long)arg1->effective,
+			(unsigned long)arg1->permitted, (unsigned long)arg1->inheritable);
+	}
+	return 0;
+}
+
+int
+sys_capset(tcp)
+struct tcb *tcp;
+{
+	cap_user_header_t       arg0;
+	cap_user_data_t         arg1;
+
+   	if(entering(tcp)) {
+		arg0 = (cap_user_header_t)tcp->u_arg[0];
+		arg1 = (cap_user_data_t)tcp->u_arg[1];
+		tprintf("{%lx, %d}, ", (unsigned long)arg0->version, arg0->pid);
+		tprintf("{%lx, %lx, %lx}", (unsigned long)arg1->effective,
+			(unsigned long)arg1->permitted, (unsigned long)arg1->inheritable);
+	}
+	return 0;
+}
+
+#else
+
+int sys_capget(tcp)
+struct tcb *tcp;
+{
+	return printargs(tcp);
+}
+
+int sys_capset(tcp)
+struct tcb *tcp;
+{
+	return printargs(tcp);
+}
+
+#endif
+
