@@ -24,14 +24,14 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: term.c,v 2.27 1996/05/21 03:20:58 jrs Exp $
+ *	$Id: term.c,v 1.1.1.1 1999/02/19 00:22:50 wichert Exp $
  */
 
 #include "defs.h"
 
-#ifndef ALPHA
+#ifdef __GLIBC__
 #include <termio.h>
-#endif
+#endif /* __GLIBC__ */
 #include <termios.h>
 #ifdef HAVE_SYS_FILIO_H
 #include <sys/filio.h>
@@ -279,10 +279,16 @@ long code, arg;
 #ifdef TIOCSETPGRP
 	case TIOCSETPGRP:
 #endif
+#ifdef FIONREAD
 	case FIONREAD:
+#endif
 	case TIOCOUTQ:
+#ifdef FIONBIO
 	case FIONBIO:
+#endif
+#ifdef FIOASYNC
 	case FIOASYNC:
+#endif
 #ifdef FIOGETOWN
 	case FIOGETOWN:
 #endif
