@@ -26,7 +26,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: util.c,v 2.38 1996/10/28 03:30:42 jrs Exp $
+ *	$Id: util.c,v 1.3 1999/03/17 00:42:25 wichert Exp $
  */
 
 #include "defs.h"
@@ -39,11 +39,56 @@
 #include <a.out.h>
 #include <link.h>
 #endif /* SUNOS4 */
+#if __GLIBC__ == 2 && __GLIBC_MINOR__ >= 1 && (defined(I386) || defined(M68K))
+# include <sys/reg.h>
+# define PTRACE_PEEKUSR PTRACE_PEEKUSER
+#endif
+#ifdef LINUX
+#include <linux/ptrace.h>
+#endif /* LINUX */
 
 #ifdef SUNOS4_KERNEL_ARCH_KLUDGE
 #include <sys/utsname.h>
 #endif /* SUNOS4_KERNEL_ARCH_KLUDGE */
 
+#if defined(LINUX) && defined(SPARC) && !defined(__GLIBC__)
+
+#include <linux/unistd.h>
+
+#define _hack_syscall5(type,name,type1,arg1,type2,arg2,type3,arg3,type4,arg4,\
+          type5,arg5,syscall) \
+type name (type1 arg1,type2 arg2,type3 arg3,type4 arg4,type5 arg5) \
+{ \
+      long __res; \
+\
+__asm__ volatile ("or %%g0, %1, %%o0\n\t" \
+                  "or %%g0, %2, %%o1\n\t" \
+                  "or %%g0, %3, %%o2\n\t" \
+                  "or %%g0, %4, %%o3\n\t" \
+                  "or %%g0, %5, %%o4\n\t" \
+                  "or %%g0, %6, %%g1\n\t" \
+                  "t 0x10\n\t" \
+                  "bcc 1f\n\t" \
+                  "or %%g0, %%o0, %0\n\t" \
+                  "sub %%g0, %%o0, %0\n\t" \
+                  "1:\n\t" \
+                  : "=r" (__res) \
+                  : "0" ((long)(arg1)),"1" ((long)(arg2)), \
+                    "2" ((long)(arg3)),"3" ((long)(arg4)),"4" ((long)(arg5)), \
+                    "i" (__NR_##syscall)  \
+                  : "g1", "o0", "o1", "o2", "o3", "o4"); \
+if (__res>=0) \
+        return (type) __res; \
+errno = -__res; \
+return -1; \
+}
+
+static _hack_syscall5(int,_ptrace,int,__request,int,__pid,int,__addr,int,__data,int,__addr2,ptrace)
+
+#define _ptrace
+
+#endif
+
 /* macros */
 #ifndef MAX
 #define MAX(a,b)		(((a) > (b)) ? (a) : (b))
@@ -471,25 +516,9 @@ char *laddr;
 {
 
 #ifdef LINUX
-#if 0
-	if (tcp->pfd == -2)
-		return -1;
-	if (tcp->pfd == -1) {
-		char buf[32];
-
-		sprintf(buf, "/proc/%d/mem", tcp->pid);
-		if ((tcp->pfd = open(buf, O_RDONLY)) == -1) {
-			perror(buf);
-			tcp->pfd = -2;
-			return -1;
-		}
-	}
-	lseek(tcp->pfd, addr, 0);
-	if (read(tcp->pfd, laddr, len) == -1)
-		return -1;
-#else
 	int pid = tcp->pid;
 	int n, m;
+	int started = 0;
 	union {
 		long val;
 		char x[sizeof(long)];
@@ -502,9 +531,15 @@ char *laddr;
 		errno = 0;
 		u.val = ptrace(PTRACE_PEEKDATA, pid, (char *) addr, 0);
 		if (errno) {
+			if (started && (errno==EPERM || errno==EIO)) {
+				/* Ran into 'end of memory' - stupid "printpath" */
+				return 0;
+			}
+			/* But if not started, we had a bogus address. */
 			perror("ptrace: umoven");
 			return -1;
 		}
+		started = 1;
 		memcpy(laddr, &u.x[n], m = MIN(sizeof(long) - n, len));
 		addr += sizeof(long), laddr += m, len -= m;
 	}
@@ -512,13 +547,17 @@ char *laddr;
 		errno = 0;
 		u.val = ptrace(PTRACE_PEEKDATA, pid, (char *) addr, 0);
 		if (errno) {
+			if (started && (errno==EPERM || errno==EIO)) {
+				/* Ran into 'end of memory' - stupid "printpath" */
+				return 0;
+			}
 			perror("ptrace: umoven");
 			return -1;
 		}
+		started = 1;
 		memcpy(laddr, u.x, m = MIN(sizeof(long), len));
 		addr += sizeof(long), laddr += m, len -= m;
 	}
-#endif
 #endif /* LINUX */
 
 #ifdef SUNOS4
@@ -607,6 +646,7 @@ char *laddr;
 #ifdef SRVR4
 	return umoven(tcp, addr, len, laddr);
 #else /* !SVR4 */
+	int started = 0;
 	int pid = tcp->pid;
 	int i, n, m;
 	union {
@@ -621,9 +661,14 @@ char *laddr;
 		errno = 0;
 		u.val = ptrace(PTRACE_PEEKDATA, pid, (char *)addr, 0);
 		if (errno) {
+			if (started && (errno==EPERM || errno==EIO)) {
+				/* Ran into 'end of memory' - stupid "printpath" */
+				return 0;
+			}
 			perror("umovestr");
 			return -1;
 		}
+		started = 1;
 		memcpy(laddr, &u.x[n], m = MIN(sizeof(long)-n,len));
 		while (n & (sizeof(long) - 1))
 			if (u.x[n++] == '\0')
@@ -634,9 +679,14 @@ char *laddr;
 		errno = 0;
 		u.val = ptrace(PTRACE_PEEKDATA, pid, (char *)addr, 0);
 		if (errno) {
+			if (started && (errno==EPERM || errno==EIO)) {
+				/* Ran into 'end of memory' - stupid "printpath" */
+				return 0;
+			}
 			perror("umovestr");
 			return -1;
 		}
+		started = 1;
 		memcpy(laddr, u.x, m = MIN(sizeof(long), len));
 		for (i = 0; i < sizeof(long); i++)
 			if (u.x[i] == '\0')
@@ -649,8 +699,10 @@ char *laddr;
 }
 
 #ifdef LINUX
+#ifndef SPARC
 #define PTRACE_WRITETEXT	101
 #define PTRACE_WRITEDATA	102
+#endif /* !SPARC */
 #endif /* LINUX */
 
 #ifdef SUNOS4
@@ -803,6 +855,14 @@ struct tcb *tcp;
 	if (upeek(tcp->pid, 4*EIP, &pc) < 0)
 		return -1;
 #else /* !I386 */
+#ifdef ARM
+	if (upeek(tcp->pid, 4*15, &pc) < 0)
+		return -1;
+#else /* !ARM */
+#ifdef POWERPC
+	if (upeek(tcp->pid, 4*PT_NIP, &pc) < 0)
+		return -1;
+#else
 #ifdef M68K
 	if (upeek(tcp->pid, 4*PT_PC, &pc) < 0)
 		return -1;
@@ -810,8 +870,17 @@ struct tcb *tcp;
 #ifdef ALPHA
 	if (upeek(tcp->pid, REG_PC, &pc) < 0)
 		return -1;
+#else /* !ALPHA */
+#ifdef SPARC
+	struct pt_regs regs;
+	if (ptrace(PTRACE_GETREGS,tcp->pid,(char *)&regs,0) < 0)
+		return -1;
+	pc = regs.pc;
+#endif /* SPARC */
 #endif /* ALPHA */
 #endif /* !M68K */
+#endif /* !POWERPC */
+#endif /* !ARM */
 #endif /* !I386 */
 	return pc;
 #endif /* LINUX */
@@ -852,6 +921,15 @@ struct tcb *tcp;
 	}
 	tprintf("[%08lx] ", eip);
 #else /* !I386K */
+#ifdef POWERPC
+	long pc;
+
+	if (upeek(tcp->pid, 4*PT_NIP, &pc) < 0) {
+		tprintf ("[????????] ");
+		return;
+	}
+	tprintf("[%08lx] ", pc);
+#else /* !POWERPC */
 #ifdef M68K
 	long pc;
 
@@ -869,8 +947,18 @@ struct tcb *tcp;
 		return;
 	}
 	tprintf("[%08lx] ", pc);
+#else /* !ALPHA */
+#ifdef SPARC
+	struct pt_regs regs;
+	if (ptrace(PTRACE_GETREGS,tcp->pid,(char *)&regs,0) < 0) {
+		tprintf("[????????] ");
+		return;
+	}
+	tprintf("[%08lx] ", regs.pc);
+#endif /* SPARC */
 #endif /* ALPHA */
 #endif /* !M68K */
+#endif /* !POWERPC */
 #endif /* !I386 */
 #endif /* LINUX */
 
@@ -900,37 +988,84 @@ struct tcb *tcp;
 {
 
 #ifdef LINUX
+#ifdef SPARC
+	/* We simply use the SunOS breakpoint code. */
 
-#ifdef I386
+	struct pt_regs regs;
+#define LOOPA	0x30800000	/* ba,a	0 */
+
+	if (tcp->flags & TCB_BPTSET) {
+		fprintf(stderr, "PANIC: TCB already set in pid %u\n", tcp->pid);
+		return -1;
+	}
+	if (ptrace(PTRACE_GETREGS, tcp->pid, (char *)&regs, 0) < 0) {
+		perror("setbpt: ptrace(PTRACE_GETREGS, ...)");
+		return -1;
+	}
+	memmove (&regs.u_regs [1], &regs.u_regs [0],
+		 sizeof (regs.u_regs) - sizeof (regs.u_regs [0]));
+	tcp->baddr = regs.u_regs[UREG_I7] + 8;
+	errno = 0;
+	tcp->inst[0] = ptrace(PTRACE_PEEKTEXT, tcp->pid, (char *)tcp->baddr, 0);
+	if(errno) {
+		perror("setbpt: ptrace(PTRACE_PEEKTEXT, ...)");
+		return -1;
+	}
+
+	/*
+	 * XXX - BRUTAL MODE ON
+	 * We cannot set a real BPT in the child, since it will not be
+	 * traced at the moment it will reach the trap and would probably
+	 * die with a core dump.
+	 * Thus, we are force our way in by taking out two instructions
+	 * and insert an eternal loop instead, in expectance of the SIGSTOP
+	 * generated by out PTRACE_ATTACH.
+	 * Of cause, if we evaporate ourselves in the middle of all this...
+	 */
+	errno = 0;
+	ptrace(PTRACE_POKETEXT, tcp->pid, (char *) tcp->baddr, LOOPA);
+	if(errno) {
+		perror("setbpt: ptrace(PTRACE_POKETEXT, ...)");
+		return -1;
+	}
+	tcp->flags |= TCB_BPTSET;
+
+#else /* !SPARC */
+
+#if defined (I386)
 #define LOOP	0x0000feeb
-#else /* !I386 */
-#ifdef M68K
+#elif defined (M68K)
 #define LOOP	0x60fe0000
-#else /* !M68K */
-#ifdef ALPHA
+#elif defined (ALPHA)
 #define LOOP    0xc3ffffff
-#endif /* ALPHA */
-#endif /* !M68K */
-#endif /* !I386 */
+#elif defined (POWERPC)
+#define LOOP 0x0000feeb
+#elif defined(ARM)
+#define LOOP -1  /* almost certainly wrong, jws */
+#else
+#error unknown architecture
+#endif
 
 	if (tcp->flags & TCB_BPTSET) {
 		fprintf(stderr, "PANIC: bpt already set in pid %u\n", tcp->pid);
 		return -1;
 	}
-#ifdef I386
+#if defined (I386)
 	if (upeek(tcp->pid, 4*EIP, &tcp->baddr) < 0)
 		return -1;
-#else /* !I386 */
-#ifdef M68K
+#elif defined (M68K)
 	if (upeek(tcp->pid, 4*PT_PC, &tcp->baddr) < 0)
 	  return -1;
-#else /* !M68K */
-#ifdef ALPHA
-	if (upeek(tcp->pid, REG_PC, &tcp->baddr) < 0)
-	  return -1;
-#endif /* ALPHA */
-#endif /* !M68K */
-#endif /* !I386 */
+#elif defined (ALPHA)
+	return -1;
+#elif defined (ARM)
+	return -1;
+#elif defined (POWERPC)
+	if (upeek(tcp->pid, 4*PT_NIP, &tcp->baddr) < 0)
+		return -1;
+#else
+#error unknown architecture
+#endif
 	if (debug)
 		fprintf(stderr, "[%d] setting bpt at %lx\n", tcp->pid, tcp->baddr);
 	tcp->inst[0] = ptrace(PTRACE_PEEKTEXT, tcp->pid, (char *) tcp->baddr, 0);
@@ -945,12 +1080,13 @@ struct tcb *tcp;
 	}
 	tcp->flags |= TCB_BPTSET;
 
+#endif /* SPARC */
 #endif /* LINUX */
 
 #ifdef SUNOS4
 #ifdef SPARC	/* This code is slightly sparc specific */
 
-	struct regs regs;
+	struct pt_regs regs;
 #define BPT	0x91d02001	/* ta	1 */
 #define LOOP	0x10800000	/* ba	0 */
 #define LOOPA	0x30800000	/* ba,a	0 */
@@ -1008,6 +1144,9 @@ struct tcb *tcp;
 #ifdef I386
 	long eip;
 #else /* !I386 */
+#ifdef POWERPC
+	long pc;
+#else /* !POWERPC */
 #ifdef M68K
 	long pc;
 #else /* !M68K */
@@ -1015,8 +1154,24 @@ struct tcb *tcp;
 	long pc;
 #endif /* ALPHA */
 #endif /* !M68K */
+#endif /* !POWERPC */
 #endif /* !I386 */
 
+#ifdef SPARC
+	/* Again, we borrow the SunOS breakpoint code. */
+	if (!(tcp->flags & TCB_BPTSET)) {
+		fprintf(stderr, "PANIC: TCB not set in pid %u\n", tcp->pid);
+		return -1;
+	}
+	errno = 0;
+	ptrace(PTRACE_POKETEXT, tcp->pid, (char *) tcp->baddr, tcp->inst[0]);
+	if(errno) {
+		perror("clearbtp: ptrace(PTRACE_POKETEXT, ...)");
+		return -1;
+	}
+	tcp->flags &= ~TCB_BPTSET;
+#else /* !SPARC */
+
 	if (debug)
 		fprintf(stderr, "[%d] clearing bpt\n", tcp->pid);
 	if (!(tcp->flags & TCB_BPTSET)) {
@@ -1043,6 +1198,17 @@ struct tcb *tcp;
 		return 0;
 	}
 #else /* !I386 */
+#ifdef POWERPC
+	if (upeek(tcp->pid, 4*PT_NIP, &pc) < 0)
+		return -1;
+	if (pc != tcp->baddr) {
+		/* The breakpoint has not been reached yet.  */
+		if (debug)
+			fprintf(stderr, "NOTE: PC not at bpt (pc %#lx baddr %#lx)\n",
+				pc, tcp->baddr);
+		return 0;
+	}
+#else /* !POWERPC */
 #ifdef M68K
 	if (upeek(tcp->pid, 4*PT_PC, &pc) < 0)
 		return -1;
@@ -1066,14 +1232,16 @@ struct tcb *tcp;
 	}
 #endif /* ALPHA */
 #endif /* !M68K */
+#endif /* !POWERPC */
 #endif /* !I386 */
+#endif /* !SPARC */
 #endif /* LINUX */
 
 #ifdef SUNOS4
 #ifdef SPARC
 
 #if !LOOPA
-	struct regs regs;
+	struct pt_regs regs;
 #endif
 
 	if (!(tcp->flags & TCB_BPTSET)) {
